#!/bin/sh
#\
exec wish $0 $*

#
# ShowWord.tcl
# args: filename word <line number where word first appears>
#

set backGround gray85
set foreGround black

tk_setPalette background $backGround foreground $foreGround \
	selectBackground $foreGround selectForeground $backGround
. configure -bg $backGround

proc Next { text word index } {
    set exp "(\[^A-Za-z0-9_\]|^)($word)(\[^A-Za-z0-9_\]|\$)"
    set index [$text index "$index + 1 chars"]
    set next [$text search -forwards -nocase -regexp \
	    $exp $index end]
    if {$next == {}} {
	bell
	return {}
    }
    set next [$text search -forwards -nocase $word $next]
    #puts "Next: index $index, next $next"
    $text see $next
    HighlightWord $text $next
    return $next
}

proc Prev { text word index } {
    set exp "(\[^A-Za-z0-9_\]|^)($word)(\[^A-Za-z0-9_\]|\$)"
    set index [$text index "$index - 1 chars"]
    set next [$text search -backwards -nocase -regexp \
	    $exp $index 1.0]
    if {$next == {}} {
	bell
	return {}
    }
    set next [$text search -forwards -nocase $word $next]
    #puts "Prev: index $index, next $next"
    $text see $next
    HighlightWord $text $next
    return $next
}

proc HighlightWord { text index } {
    global Index Line
    $text tag delete TheWord
    $text tag add TheWord $index "$index wordend"
    $text tag config TheWord -foreground black -background yellow
    set Index $index
    set Line "Line [lindex [split $index .] 0]"
}

proc Build {} {
    
    global Word

    foreach w [winfo children .] {
	catch {destroy $w}
    }
    
    . config -relief flat -bd 4 -highlightthickness 3 \
	    -highlightbackground black
    
    frame .top
    set f [frame .top.file]
    label $f.label -anchor w -text "File:"
    label $f.value -anchor w -textvariable FileTail -relief sunken -bd 2
    pack $f.label -side left
    pack $f.value -side left -fill x -expand true
    pack $f -side left -fill x -expand true
    
    if {$Word != {}} {
	set f [frame .top.string]
	label $f.label -anchor w -text "Search string:"
	label $f.value -anchor w -textvariable Word -relief sunken -bd 2
	pack $f.label -side left
	pack $f.value -side left -fill x -expand true
	pack $f -side right -fill x -expand true
    }
        
    set f [frame .text]
    set t [text $f.t -setgrid true -wrap none -state normal -font fixed \
	    -exportselection false -highlightthickness 0 \
	    -yscrollcommand "$f.sy set" -xscrollcommand "$f.sx set"]
    scrollbar $f.sy -orient vertical -width 10 -command "$f.t yview" \
	    -highlightthickness 0
    scrollbar $f.sx -orient horizontal -width 10 -command "$f.t xview" \
	    -highlightthickness 0    
    pack $f.sy -side right -fill y
    pack $f.sx -side bottom -fill x
    pack $f.t -side left -anchor nw -fill both -expand true    

    pack .top -side top -fill x -anchor nw -ipady 2 -pady 2 -padx 2
    pack .text -side top -fill both -expand true

    if {$Word != {}} {
	frame .bottom    
	set f [frame .bottom.lineno]
	label $f.lineno -relief sunken -bd 2 -textvariable Line
	pack $f.lineno -fill x -expand true
	pack $f -side left -fill x -padx 10 -expand true
	set f [frame .bottom.buttons]
	button $f.first -text "|<\nFirst" -font fixed\
		-command "Next $t \$Word 0.0"
	button $f.prev -text "  <  \nPrev" -font fixed \
		-command "Prev $t \$Word \$Index"
	button $f.next -text "  >  \nNext" -font fixed \
		-command "Next $t \$Word \$Index"
	button $f.last -text "  >| \nLast" -font fixed \
		-command "Prev $t \$Word end"
	pack $f.first $f.prev $f.next $f.last -side left
	pack $f -side right -fill x
	pack .bottom -side bottom -fill x
	pack .top -ipady 0
	pack .text -pady 2
    }

}

proc ShowFile { text filename } {
    global FileName UseWWW
    $text delete 1.0 end
    set FileName $filename
    set oldCursor [$text cget -cursor]
    if {$filename == {}} {
	return
    }
    $text config -state normal
    $text config -cursor watch
    if $UseWWW {
	set cmd cat_url
	$text insert end "\n    LOADING URL...\n\n        (this could take a while)"
    } else {
	set cmd cat
    }
    update
    if [catch {exec $cmd $filename} result] {
	$text delete 1.0 end
	$text insert end "\n*** ERROR LOADING FILE ***\n\n$result"
    } else {
	$text delete 1.0 end
	$text insert end $result
    }
    $text config -state disabled -cursor $oldCursor
}

proc Main {} {
    
    global argc argv FileName FileTail Index Word Text UseWWW
    
    set FileName [lindex $argv 0]
    set Word [lindex $argv 1]
    set Index 1.0
    set Text .text.t
    if [string match http://* $FileName] {
	if {[string length $FileName] > 40} {
	    set FileTail http://.../[file tail $FileName]
	} else {
	    set FileTail $FileName
	}
	set UseWWW 1
    } else {
	set FileTail [file tail $FileName]
	set UseWWW 0
    }
    
    Build
    
    ShowFile $Text $FileName

    if {$Word != {}} {
	Next $Text $Word 1.0
    }
    
}

#
# Begin executing...
#

if {![info exists tcl_interactive] || !$tcl_interactive} {
    Main
} else {
    puts "You can begin execution by submitting the Main command"
}

